local AssistAssist = LibStub("AceAddon-3.0"):GetAddon("AssistAssist")
local Config = AssistAssist:GetModule("Config")
local AceConfig = LibStub("AceConfig-3.0")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")
local AceDBOptions = LibStub("AceDBOptions-3.0")

-- Utility function to make it easy to put headers on paragraphs
local textInABox = function(title, text, order) 
	return {
		type = "group",
		name = title,
		inline = true,
		order = order,
		args = {
			text = {
				type = "description",
				fontSize = "medium",
				name = text
			}
		}
	}
end

-- Ace options table
local options = {
	type = "group",
	name = "AssistAssist",
	args = {
		key = {
			name = "Assist/mark",
			type = "keybinding",
			desc = "Set the keybinding to "..string.lower(_G["BINDING_NAME_CLICK AssistAssistSB:LeftButton"]),
			order = 10,
			set = function(info, val)
				-- unbind old key
				local oldKey = GetBindingKey("CLICK AssistAssistSB:LeftButton")
				if oldKey ~= nil then
					SetBinding(oldKey, nil)
				end
				-- set new one if it wasn't escape which is supposed to just clear
				if key ~= "ESCAPE" then
					SetBindingClick(val, "AssistAssistSB")
				end
				-- save
				SaveBindings(GetCurrentBindingSet())
			end,
			get = function(info)
				return GetBindingKey("CLICK AssistAssistSB:LeftButton")
			end
		},
		setkey = {
			name = "Manually set",
			type = "keybinding",
			desc = "Set the keybinding to "..string.lower(BINDING_NAME_AASETASSIST),
			order = 11,
			set = function(info, val)
				-- unbind old key
				local oldKey = GetBindingKey("AASETASSIST")
				if oldKey ~= nil then
					SetBinding(oldKey, nil)
				end
				-- set new one if it wasn't escape which is supposed to just clear
				if key ~= "ESCAPE" then
					SetBinding(val, "AASETASSIST")
				end
				-- save
				SaveBindings(GetCurrentBindingSet())
			end,
			get = function(info)
				return GetBindingKey("AASETASSIST")
			end
		},
		mark = {
			order = 20,
			name = "Mark to apply as tank",
			desc = "What mark to apply when you're the tank",
			type = "select",
			values = {
				["1"] = "Star",
				["2"] = "Circle",
				["3"] = "Diamond",
				["4"] = "Triangle",
				["5"] = "Moon",
				["6"] = "Square",
				["7"] = "X",
				["8"] = "Skull"
			},
			set = function(info, val) 
				AssistAssist.db.profile.mark = val 
				AssistAssist:updateIfNecessary()
			end,
			get = function(info) return AssistAssist.db.profile.mark end
		},
		noTank = {
			order = 21,
			name = "Action with no tank",
			desc = "What to do if there isn't a tank",
			type = "select",
			values = {
				mark = "Apply mark",
				nothing = "Do nothing",
			},
			set = function(info, val)
				AssistAssist.db.profile.noTank = val
				AssistAssist:updateIfNecessary()
			end,
			get = function(info) return AssistAssist.db.profile.noTank end
		},
		help = textInABox(
			"Usage",
			[[The "Assist/mark" keybinding will automatically assist the dungeon finder tank or mark the current target if that's you.  If there are multiple tanks, it will always mark if you are one of them and assist one if you aren't; the "Manually set" keybinding allows you to pick which one.

The "Manually set" keybinding will allow you to set your current target as the person to assist; this will result in marking if you use it with yourself targetted.  If you manually set a tank/assist, it will not automatically update (for example if your tank has dropped and the dungeon finder gets you a new one) until you use it again without a target, which returns AssistAssist to auto mode. 

The "Mark to apply as tank" dropdown chooses what mark to apply when you are the tank/person to assist.  The "Action with no tank" lets you choose to apply marks by default when there isn't a tank or to do nothing.

If you have a LibDataBroker display addon, AssistAssist provides a plugin which will let you see what it is currently set to do, click to manually set, and right click to open preferences.]],
			30
		),
		about = textInABox(
			"About", 
			[[AssistAssist by Cayenne, version 1.0.

You can contact me via the AssistAssist page on curse or by emailing me at luacayenne@gmail.com.  Let me know if you find bugs: the way things are, chances are multiple people are experiencing it but no one lets me know.  Also let me know if you'd like additional features:  I like to add them when I know someone would use them.]], -- TODO: remember to manually change version number with each update
			 31
		)
	}
}

local defaults = {
	profile = {
		mark = "8",
		noTank = "nothing"
	}
}

function Config:OnInitialize()
	-- saved preferences
	AssistAssist.db = LibStub("AceDB-3.0"):New("AssistAssistDB", defaults, true)
	-- main options panel 
	AceConfig:RegisterOptionsTable("AssistAssist", options) 
	AceConfigDialog:AddToBlizOptions("AssistAssist")
end
